/*
  Copyright (C) 2013-2014 Stefano Verzegnassi

    This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License 3 as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

    This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
  along with this program. If not, see http://www.gnu.org/licenses/.
*/

#ifndef STORAGEMANAGER_H
#define STORAGEMANAGER_H

#include <QFile>
#include <QDir>
#include <QDirIterator>

class StorageManager : public QObject
{
    Q_OBJECT
    Q_ENUMS(XdgFolders)

public:
    StorageManager(QObject *parent = 0);

    enum XdgFolders {
        HomeLocation,
        CacheLocation,
        DataLocation,
        ConfigLocation
    };

    // Get user folders
    Q_INVOKABLE QString getXdgFolder(StorageManager::XdgFolders folder);

    // operations
    Q_INVOKABLE bool exists(QString path);
    Q_INVOKABLE bool mkPath(QString path);
    Q_INVOKABLE bool rename(QString completePathFile, QString newBaseName);
    Q_INVOKABLE bool rm(QString path);
    Q_INVOKABLE bool copy(QString sourcePath, QString destDir);

    Q_INVOKABLE QStringList searchRecursively(QString path, QStringList filters);

    Q_INVOKABLE QString getFileBaseNameFromPath(QString path);
    Q_INVOKABLE QString getFileCompleteSuffixFromPath(QString path);

    Q_INVOKABLE bool isFile(QString path);
    Q_INVOKABLE bool isDir(QString path);

private:
    bool copyPath(QString source, QString destination);
};

#endif // STORAGEMANAGER_H

